package tv.younify.sdk.connectapp

import android.os.Bundle
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.content.Intent
import android.net.Uri
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.lifecycle.lifecycleScope
import androidx.navigation.findNavController
import androidx.navigation.ui.AppBarConfiguration
import androidx.navigation.ui.navigateUp
import androidx.navigation.ui.setupActionBarWithNavController
import kotlinx.coroutines.launch
import org.json.JSONObject
import tv.younify.sdk.connect.Connect
import tv.younify.sdk.connect.ConnectOptions
import tv.younify.sdk.connect.LogLevel
import tv.younify.sdk.connect.LogListener
import tv.younify.sdk.connect.RenewTokensCallback
import tv.younify.sdk.connect.TokenHandler
import tv.younify.sdk.connectapp.databinding.ActivityMainBinding

class MainActivity : AppCompatActivity(), TokenHandler, LogListener {
    private lateinit var appBarConfiguration: AppBarConfiguration
    private lateinit var binding: ActivityMainBinding
    private val servicesViewModel: YounifyServicesViewModel by viewModels()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        initConnectSDK()

        binding = ActivityMainBinding.inflate(layoutInflater)

        val view = binding.root
        setContentView(view)

        setSupportActionBar(binding.toolbar)

        val navController = findNavController(R.id.nav_host_fragment_content_main)
        appBarConfiguration = AppBarConfiguration(navController.graph)
        setupActionBarWithNavController(navController, appBarConfiguration)

        intent?.data?.let {
            handleLink(it)
        }
    }

    private fun initConnectSDK() {
        val tokens = TokenManager.loadTokens(this)
        val options = ConnectOptions(
            key = "_YOUR_APP_KEY_HERE_", //TODO: Set your app key.
            logLevel = LogLevel.Warning,
            logListener = this,
            tokenHandler = this,
            accessToken = tokens.accessToken,
            refreshToken = tokens.refreshToken,
            extra = null
        )

        Connect.configure(options)
    }

    private fun handleLink(uri: Uri): Boolean {
        //TODO: Remove the following block and scheme based intent-filter in AndroidManifest.xml for production apps!
        //The following block allows configuring the Younify Connect SDK from a url in order to facilitate testing.
        //The url format is ycssampleapp://configure/?key=<sdk key>&at=<user's access token>&rt=<user's refresh token>.
        if (uri.scheme == "ycssampleapp") {
            if (uri.host == "configure") {
                val key = uri.getQueryParameter("key")
                val at = uri.getQueryParameter("at")
                val rt = uri.getQueryParameter("rt")
                val extra = uri.getQueryParameter("extra")

                var extraMap: Map<String, Any>? = null
                if (extra != null) {
                    runCatching {
                        val extraJSONObject = JSONObject(extra)
                        extraMap = mutableMapOf<String, Any>().apply {
                            extraJSONObject.keys().forEach { put(it, extraJSONObject[it]) }
                        }
                    }
                }

                if ((key != null) && (at != null) && (rt != null)) {
                    val options = ConnectOptions(
                        key = key,
                        logLevel = LogLevel.Warning,
                        logListener = this,
                        tokenHandler = null, //XXX: Because the key isn't being persisted, prevent token renewals from being persisted as well.
                        accessToken = at,
                        refreshToken = rt,
                        extra = extraMap
                    )

                    Connect.configure(options)
                    lifecycleScope.launch {
                        runCatching {
                            servicesViewModel.refresh()
                        }
                    }

                    return true
                }
            }
        }

        return false
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        // Inflate the menu; this adds items to the action bar if it is present.
        menuInflater.inflate(R.menu.menu_main, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        return when (item.itemId) {
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun onSupportNavigateUp(): Boolean {
        val navController = findNavController(R.id.nav_host_fragment_content_main)
        return navController.navigateUp(appBarConfiguration) || super.onSupportNavigateUp()
    }

    override fun onNewIntent(newIntent: Intent?) {
        super.onNewIntent(newIntent)

        if (newIntent?.action == Intent.ACTION_VIEW) {
            newIntent.data?.let {
                handleLink(it)
            }
        }
    }

    override fun log(level: LogLevel, message: String) {
        val priority = when (level) {
            LogLevel.Trace -> Log.VERBOSE
            LogLevel.Debug -> Log.DEBUG
            LogLevel.Information -> Log.INFO
            LogLevel.Warning -> Log.WARN
            LogLevel.Error -> Log.ERROR
            LogLevel.Critical -> Log.ASSERT
            else -> return
        }

        Log.println(priority, "YCSSA", message)
    }

    override fun renewTokens(expiredAccessToken: String?, refreshToken: String?, complete: RenewTokensCallback) {
        //TODO: Implement this method to perform a backend token renewal by proxying the request
        //to your own backend servers, then to Younify using your private api key.
        complete(null, null)
    }

    override fun renewedTokens(newAccessToken: String, newRefreshToken: String) {
        TokenManager.storeTokens(this, newAccessToken, newRefreshToken)
    }
}
