package tv.younify.sdk.connectapp

import android.app.AlertDialog
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.fragment.app.activityViewModels
import androidx.fragment.app.commit
import androidx.fragment.app.replace
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import kotlinx.coroutines.launch
import tv.younify.sdk.connect.StreamingContent
import tv.younify.sdk.connect.StreamingContentType
import tv.younify.sdk.connect.StreamingService
import tv.younify.sdk.connectapp.databinding.FragmentYounifyBrowseBinding
import kotlin.coroutines.cancellation.CancellationException

private const val ARG_CONTENT = "content"
private const val ARG_CONTENT_SERVICE = "contentService"
private const val ARG_SERVICES = "services"
private const val ARG_SEARCHTERM = "searchTerm"
private const val ARG_ISSEARCH = "isSearch"

class YounifyBrowseFragment : Fragment(), ImageRequestListener {
    private var _binding: FragmentYounifyBrowseBinding? = null
    private val binding get() = _binding!!

    private var streamingContent: StreamingContent? = null
    private var contentService: StreamingService? = null
    private var services: List<StreamingService>? = null
    private var searchTerm: String? = null
    private var isSearch: Boolean = false
    private lateinit var adapter: YounifyContentItemsAdapter
    private lateinit var contentItemsRecyclerView: RecyclerView

    private val servicesViewModel: YounifyServicesViewModel by activityViewModels()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.UPSIDE_DOWN_CAKE) {
                streamingContent = it.getParcelable(ARG_CONTENT, StreamingContent::class.java)
                contentService = it.getParcelable(ARG_CONTENT_SERVICE, StreamingService::class.java)
                services = it.getParcelableArrayList(ARG_SERVICES, StreamingService::class.java)
            } else {
                @Suppress("DEPRECATION")
                streamingContent = it.getParcelable(ARG_CONTENT)

                @Suppress("DEPRECATION")
                contentService = it.getParcelable(ARG_CONTENT_SERVICE)

                @Suppress("DEPRECATION")
                services = it.getParcelableArrayList(ARG_SERVICES)
            }

            searchTerm = it.getString(ARG_SEARCHTERM)
            isSearch = it.getBoolean(ARG_ISSEARCH)
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        binding.activityIndicator.visibility = View.VISIBLE
        contentItemsRecyclerView = view.findViewById(R.id.content_items_recycler_view)
        contentItemsRecyclerView.layoutManager = GridLayoutManager(this.context, 1, GridLayoutManager.VERTICAL, false)

        adapter = YounifyContentItemsAdapter(lifecycleScope, YounifyContentItemsAdapter.OnTapListener { content: StreamingContent, service: StreamingService ->
            this.activity?.title = content.title

            val bundle = Bundle()
            bundle.putParcelable(ARG_CONTENT, content)
            bundle.putParcelable(ARG_CONTENT_SERVICE, service)
            if (content.type == StreamingContentType.Folder) {
                val servicesList = services
                bundle.putParcelableArrayList(ARG_SERVICES, if (servicesList != null) ArrayList(servicesList) else ArrayList())
                bundle.putString(ARG_SEARCHTERM, "")
                bundle.putBoolean(ARG_ISSEARCH, false)

                this.activity?.supportFragmentManager?.commit {
                    replace<YounifyBrowseFragment>(
                        R.id.fragment_container_younify_browse_view,
                        args = bundle
                    )
                    setReorderingAllowed(true)
                    addToBackStack(null)
                }
            } else {
                this.activity?.supportFragmentManager?.commit {
                    replace<YounifyDetailsFragment>(
                        R.id.fragment_container_younify_browse_view,
                        args = bundle
                    )
                    setReorderingAllowed(true)
                    addToBackStack(null)
                }
            }
        }, this)

        contentItemsRecyclerView.adapter = adapter
    }

    override suspend fun fetchImage(url:String) : ByteArray? {
        return servicesViewModel.fetchImage(url)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        _binding = FragmentYounifyBrowseBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    override fun onStart() {
        super.onStart()
        lifecycleScope.launch {
            try {
                if (isSearch)
                    search()
                else
                    browse()
            }
            catch (ex: CancellationException) {
                throw ex
            }
            catch (cause: Throwable) {
                AlertDialog.Builder(context)
                    .setMessage("Error: ${cause.message}")
                    .setTitle("Error")
                    .setPositiveButton("OK") { _, _ -> }
                    .create()
                    .show()
            }
        }
    }

    private suspend fun browse() {
        try {
            val res = servicesViewModel.fetchContentChildren(streamingContent!!)
            if (res.isNotEmpty())
                adapter.addItems(res, contentService!!)
        }
        finally {
            binding.activityIndicator.visibility = View.GONE
        }
    }

    private suspend fun search() {
        try {
            services?.let { services ->
                val res = servicesViewModel.searchContent(services, searchTerm!!)
                if (res.isNotEmpty()) {
                    res.forEach { serviceResult ->
                        adapter.addItems(serviceResult.content, serviceResult.service)
                    }
                }
            }
        }
        finally {
            binding.activityIndicator.visibility = View.GONE
        }
    }
}
