package tv.younify.sdk.connectapp

import android.app.AlertDialog
import android.graphics.BlendMode
import android.graphics.BlendModeColorFilter
import android.graphics.PorterDuff
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.activityViewModels
import androidx.fragment.app.commit
import androidx.fragment.app.replace
import androidx.lifecycle.lifecycleScope
import kotlinx.coroutines.launch
import tv.younify.sdk.connect.StreamingService
import tv.younify.sdk.connectapp.databinding.FragmentYounifyServicesBinding
import kotlin.coroutines.cancellation.CancellationException

class YounifyServicesFragment : Fragment() {
    private var _binding: FragmentYounifyServicesBinding? = null
    private val binding get() = _binding!!

    private lateinit var adapter: YounifyServicesAdapter
    private val servicesViewModel: YounifyServicesViewModel by activityViewModels()

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        _binding = FragmentYounifyServicesBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        servicesViewModel.services.observe(viewLifecycleOwner) { services ->
            val allServices = services.map { it.value!! }
            if (allServices.any { service -> service.link != null })
                binding.browseAll.visibility = View.VISIBLE

            adapter.setServiceItems(allServices)

            services.forEach {
                it.observe(viewLifecycleOwner) { service -> adapter.updateService(service) }
            }
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
            binding.loadingBar.indeterminateDrawable.colorFilter = BlendModeColorFilter(ContextCompat.getColor(requireContext(), R.color.ycs_main_color), BlendMode.SRC_ATOP)
        }
        else {
            @Suppress("DEPRECATION")
            binding.loadingBar.indeterminateDrawable.setColorFilter(ContextCompat.getColor(requireContext(), R.color.ycs_main_color), PorterDuff.Mode.SRC_ATOP)
        }

        adapter = YounifyServicesAdapter(YounifyServicesAdapter.OnTapListener { service, action ->

            lifecycleScope.launch {
                try {
                    if (action == YounifyServicesAdapter.TapListenerAction.ItemClick) {
                        val serviceLink = service.link
                        if ((serviceLink != null) && !serviceLink.isBroken)
                            servicesViewModel.manageLinkedService(service, requireContext(), requireActivity().activityResultRegistry)
                        else
                            servicesViewModel.linkService(service, requireContext(), requireActivity().activityResultRegistry)
                    } else if (action == YounifyServicesAdapter.TapListenerAction.UnlinkClick)
                        servicesViewModel.unlinkService(service)
                    else if (action == YounifyServicesAdapter.TapListenerAction.FetchClick) {
                        val bundle = Bundle()
                        bundle.putParcelableArrayList("services", arrayListOf(service))

                        activity?.supportFragmentManager?.commit {
                            setCustomAnimations(R.anim.slide_in, 0)
                            replace<YounifyRootView>(
                                R.id.nav_host_fragment_content_main,
                                args = bundle
                            )
                            setReorderingAllowed(true)
                            addToBackStack(null)
                        }
                    }
                }
                catch (ex: CancellationException) {
                    throw ex
                }
                catch (cause: Throwable) {
                    AlertDialog.Builder(requireContext())
                        .setMessage("Error: ${cause.message}")
                        .setTitle("Error processing service")
                        .setPositiveButton("OK") { _, _ -> }
                        .create()
                        .show()
                }
            }
        })

        binding.browseAll.setOnClickListener {
            val bundle = Bundle()
            val linkedServices = getLinkedServices()
            bundle.putParcelableArrayList("services", if (linkedServices != null) ArrayList(linkedServices) else null)

            activity?.supportFragmentManager?.commit {
                setCustomAnimations(R.anim.slide_in, 0)
                replace<YounifyRootView>(
                    R.id.nav_host_fragment_content_main,
                    args = bundle
                )
                setReorderingAllowed(true)
                addToBackStack(null)
            }
        }

        binding.recyclerView.adapter = adapter

        lifecycleScope.launch {
            loadServices()
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    private suspend fun loadServices() {
        try {
            binding.loadingBar.visibility = View.VISIBLE
            binding.recyclerView.visibility = View.GONE
            binding.browseAll.visibility = View.GONE

            servicesViewModel.fetchServices()

            binding.loadingBar.visibility = View.GONE
            binding.recyclerView.visibility = View.VISIBLE
        }
        catch (ex: CancellationException) {
            throw ex
        }
        catch (cause: Throwable) {
            AlertDialog.Builder(requireContext())
                .setMessage("Error: ${cause.message}")
                .setTitle("Error")
                .setPositiveButton("OK") { _, _ -> }
                .create()
                .show()

            binding.loadingBar.visibility = View.GONE
            binding.recyclerView.visibility = View.VISIBLE
        }
    }

    private fun getLinkedServices(): List<StreamingService>? {
        return servicesViewModel.services.value?.map { it.value!! }?.filter { service -> service.link != null }
    }
}
